//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Variable-width multiplier made strictly out of AND Gates and adders according to the multiplication algorithm.
//Care must be taken when reading the result due to the considerable delay of the array when using large widths.
//Version 2.0
//Date: 7-Feb-2011
//Written by: Jose Pablo Pinilla
//Investigation Group: Advanced Digital Technologies (ADT)
//Universidad Pontificia Bolivariana Seccional Bucaramanga, Colombia
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

module multiplier_top # (parameter WIDTH = 4) //This parameter specifies the width of both operands
	(
	input  [WIDTH-1:0] A, B, 
	output [2*WIDTH-1:0] product
	);


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

wire [WIDTH:0]stage[WIDTH-2:0];

wire [WIDTH:0]stagein0 = {1'b0,(A&({(WIDTH){B[0]}}))};

genvar h;

generate
	for (h=0; h<WIDTH-1; h=h+1)
	
		begin: ADDER_STAGES
			if (h==0)
				multiplier_stage #(WIDTH)	stages_inst0	(.A(A),.B(B[h+1]),.stagein(stagein0),.stage(stage[0]));
			else
				multiplier_stage #(WIDTH)	stages_inst		(.A(A),.B(B[h+1]),.stagein(stage[h-1]),.stage(stage[h]));

			assign product[0] = A[0] & B[0];
			
			assign product[h+1] = stage[h][0];
			
			assign product [(2*WIDTH-1):(2*WIDTH-1-WIDTH)] = stage[WIDTH-2];
			
		end

endgenerate

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

endmodule
