//===================================RISCKER====================================
//------------------------------------------------------------------------------
//File Name:	RISCKER_Observer
//Purpose:		RISCKER instantiation with state signals connected to the UART module
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
`timescale 1ns / 1ps

module RISCKER_Observer (
	input		CLK_50M,
	output reg[7:0] LED,
	input	[1:0] KEY,
	input [3:0] SW,
	input Rx,
	output Tx
	);

//////////////////////////////////////////////////////////////////
//======================Configuration Byte========================
//////////////////////////////////////////////////////////////////
wire [7:0]conf_byte;
reg [25:0] clk_divider;

always @ (*)
case (conf_byte[2:0])
	0:		clk_divider	=	50000000;	//0.5Hz
	1:		clk_divider	=	25000000;	//1Hz
	2:		clk_divider	=	12500000;	//2Hz
	3:		clk_divider =	5000000;		//5Hz
	4:		clk_divider =	2500000;		//10Hz
	5:		clk_divider =	1250000;		//20Hz
	6:		clk_divider =	250000;		//100Hz
	7:		clk_divider =	25000;		//1kHz
endcase
	
//////////////////////////////////////////////////////////////////
//=================Clock Frequency Division=======================
//////////////////////////////////////////////////////////////////
wire div_clk;
wire Rx_Last_Byte;
wire clk_reset = Rx_Last_Byte|conf_byte[3];

Generate_Arbitrary_Divided_Clk32 clk_div(
	.inclk(CLK_50M),
	.outclk(div_clk),
	.div_clk_count(clk_divider),
	.reset(clk_reset)
	);

//////////////////////////////////////////////////////////////////
//=======================RISCKER Processor========================
//////////////////////////////////////////////////////////////////
wire [15:0] input_port;
wire [15:0] output_port;
wire [207:0] observer_state;

datapath RISC_inst (
   .clk(div_clk), 
	.reset(KEY[0]),
   .input_port(input_port),
   .output_port(output_port), 
   .observer_state(observer_state)
   );

//////////////////////////////////////////////////////////////////
//=======================UART Observer========================
//////////////////////////////////////////////////////////////////

	UART_top UART_inst(
	.clk(~CLK_50M),
	.DataOut(observer_state),
	.Tx(Tx),
	.Rx(Rx),
	.DataIn({conf_byte,input_port}),
	.Trigger(~div_clk),
	.Rx_Last_Byte(Rx_Last_Byte)
	);
	
	defparam UART_inst.RxBytes = 3;		//	{ConfigByte[7:0],InputPort[15:0]}
												//	4		2	2	2	 2	  2	2	 2	  4	2
	defparam UART_inst.TxBytes = 26;	//	TMRs	RC	OP	SP  RG1 RG0 MWr MDa ALU Ins PC
												
	defparam UART_inst.baud_rate = 115200;

always @ *
case (SW)
	0: LED <= conf_byte;
	1: LED <= {8{div_clk}};
	2: LED <= input_port[7:0];
	3: LED <= input_port[15:8];
	4: LED <= observer_state[39:32];
	5: LED <= observer_state[47:40];
	6: LED <= observer_state[55:48];
	7: LED <= observer_state[63:56];
	default: LED <= observer_state[7:0];
endcase

endmodule 