//===================================RISCKER====================================
//=====================================ALU======================================
//------------------------------------------------------------------------------
//File Name:	alu.v
//Purpose:		Basic Arithmetic Logic Unit
//Parameters:	N (data width)
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module alu #(parameter N = 16) (a, b, aluop, y, zero, cout, sign );

input [N-1:0] a, b;
input [2:0] aluop;
output reg[N-1:0] y;
output zero, cout;
output sign;

reg N0=1'b0;
reg N1=1'b0;
reg Cin=1'b0;
reg [1:0]sel;

always @(*)
	case(aluop)
		3'b000:	{N0,N1,Cin,sel} = 5'b00000; //add
		3'b001:	{N0,N1,Cin,sel} = 5'b00001; //and
		3'b010:	{N0,N1,Cin,sel} = 5'b00010; //or
		3'b011:	{N0,N1,Cin,sel} = 5'b00011; //xor
		3'b100:	{N0,N1,Cin,sel} = 5'b11010; //nand
		3'b101:	{N0,N1,Cin,sel} = 5'b11001; //nor
		3'b110:	{N0,N1,Cin,sel} = 5'b01100; //sub, neg
		3'b111:	{N0,N1,Cin,sel} = 5'b00100; //inc
	endcase

//ALU
wire [N-1:0] a_sel = N0 ? ~a : a;
wire [N-1:0] b_sel = N1 ? ~b : b;
//Operations
wire [N:0] sum = a_sel + b_sel + Cin;
wire [N-1:0] and_gate = a_sel & b_sel;
wire [N-1:0] or_gate = a_sel | b_sel;
wire [N-1:0] xor_gate = a_sel ^ b_sel;


always@(*)
	case (sel)
		2'b00: y <= sum[N-1:0];
		2'b01: y <= and_gate;
		2'b10: y <= or_gate;
		2'b11: y <= xor_gate;
		default y <= 0;
	endcase

assign sign = y[N-1];
assign zero = (y==0);

assign cout = sum[N];

endmodule
