//==============================RISCKER Observer================================
//===============================Register File==================================
//------------------------------------------------------------------------------
//File Name:	regfile.v
//Purpose:		Register file generate loop
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------

module regfile (	input rf_write,intoggle,ipc_write,
	input [5:0] r1, r2, wr,
	input [15:0] wd,
	input [15:0] ip,tmr1,tmr2,ipc_in,
	output [15:0] rd1, rd2,
	output [15:0] pr1,pr2,ti1,ti2,eir1,eir2,op,rc,
	output [15:0] rg0,rg1,sp //Observer State signals
	);
					

//=====================64 Registers=============================
//This is how the registers are organized from 0 to 63
/*0			r0 is always 0*/		
/*1-59		General Registers*/ reg [15:0] Rfile[59:1];
//=======Read-Only Registers==============
/*60		Timer1 Value	(input tmr1)*/
/*61		Timer2 Value	(input tmr2)*/
/*62		Input Port		(input ip)*/
/*63		Interruption PC(input ipc_in)*/


/*===================Simulation Initialization===============================
(Optional) Initial.dat is an array of 0's that initialize the register file
initial
	begin
		$readmemb ("initial.dat",Rfile);
	end
*/
//=====================Register File Writing=======================
always @ (posedge rf_write or posedge intoggle)
	begin
//Intoggle clears all interruption flags on control register[3:0]
		if (intoggle)
			Rfile[59] <= {Rfile[59][15:4],4'h0};	
		else
			Rfile[wr] <= wd;
	end

//=================Interruption PC special condition===============
reg [15:0]interrupt_pc = 0;

always @ (posedge ipc_write)
	interrupt_pc <= ipc_in;		//always @ ipc_write pc is stored


//============Bus array from which to select the outputs===========

wire [15:0] rf[63:0];

//Assignment for the 64 registers

assign rf[0] = 16'd0;

 genvar i;
   generate
      for (i=1; i < 64; i=i+1) 
      begin: assignments
         if (i==60) //Timer1 Value
				assign rf[i] = tmr1;
			else
			if (i==61) //Timer2 Value
				assign rf[i] = tmr2;
			else
			if (i==62) //Input Port Synchronized
				assign rf[i] = ip;
			else
			if (i==63) //Interruption Program Counter
				assign rf[i] = interrupt_pc;
			else 
				assign rf[i] = Rfile[i];
      end
   endgenerate
		

//================Output Selection=====================

assign rd1 = rf[r1];		//Read data 1
assign rd2 = rf[r2];		//Read data 2

//Special Purpouse Registers are connected outside of the RegFile

assign pr1 = rf[52];		//Timer1 period
assign pr2 = rf[53];		//Timer2 period
assign ti1 = rf[54];		//Timer1 interruption address
assign ti2 = rf[55];		//Timer2 interruption address
assign eir1 =rf[56];		//External interruption 1 address
assign eir2 =rf[57];		//External interruption 2 address
assign op 	=rf[58];		//Output port
assign rc 	=rf[59];		//Control register
///////////////////////////////////////////////////////
//Observer Status Signals
assign rg0 = rf[25];
assign rg1 = rf[26];
assign sp = rf[51];
///////////////////////////////////////////////////////

endmodule
